@@ -6,6 +6,7 @@ from __future__ import annotations
 
 import logging
 import math
+import re
 import textwrap
 import warnings
 import xml.etree.ElementTree as ET
@@ -435,7 +436,7 @@ class _NumOrRefIndexer(Generic[G]):
         self._list += val
 
 
-class CustomStep(ProtoCommand):
+class CustomStep(ProtoCommand, XMLable):
     """A protocol step composed of SCPI/protocol commands."""
 
     _body: list[ProtoCommand]
@@ -494,6 +495,23 @@ class CustomStep(ProtoCommand):
     def collects(self) -> bool:
         return False
 
+    def to_xml(self, **kwargs: Any) -> ET.Element:
+        assert not kwargs
+        e = ET.Element("TCStep")
+        ET.SubElement(e, "CollectionFlag").text = str(
+            int(self.collects)
+        )  # FIXME: approx
+        for t in range(0, 6):  # FIXME
+            ET.SubElement(e, "Temperature").text = "30.0"
+        ET.SubElement(e, "HoldTime").text = "1"
+        # FIXME: does not contain cycle starts, because AB format can't handle
+        ET.SubElement(e, "ExtTemperature").text = str(0)
+        ET.SubElement(e, "ExtHoldTime").text = str(0)
+        # FIXME: RampRate, RampRateUnit
+        ET.SubElement(e, "RampRate").text = "1.6"
+        ET.SubElement(e, "RampRateUnit").text = "DEGREES_PER_SECOND"
+        return e
+
     def to_scpicommand(self, *, stepindex: int = -1, **kwargs: Any) -> SCPICommand:
         opts = {}
         args: list[int | str | None | Sequence[SCPICommand]] = []
@@ -639,9 +657,15 @@ class Step(CustomStep, XMLable):
     def info_str(self, index: None | int = None, repeats: int = 1) -> str:
         "String describing the step."
 
-        tempstr = "{:.2f~}".format(self.temperatures_at_cycle(1))  # type: ignore
+        temperatures_cycle1 = self.temperatures_at_cycle(1)
+        temperatures_cycle1 = convert_quantity_ndarray_to_scalar_if_all_equal(
+            temperatures_cycle1
+        )
+        tempstr = "{:.2f~}".format(temperatures_cycle1)  # type: ignore
         if (repeats > 1) and (self.temp_increment != 0.0):
-            t = "{:.2f~}".format(self.temperatures_at_cycle(repeats))  # type: ignore
+            temperatures = self.temperatures_at_cycle(repeats)
+            temperatures = convert_quantity_ndarray_to_scalar_if_all_equal(temperatures)
+            t = "{:.2f~}".format(temperatures)  # type: ignore
             tempstr += f" to {t}"
 
         elems = [f"{tempstr} for {self.time}/cycle"]
@@ -846,6 +870,25 @@ class Step(CustomStep, XMLable):
         return cls(**d)
 
 
+def convert_quantity_ndarray_to_scalar_if_all_equal(
+    quants: pint.Quantity,
+) -> pint.Quantity:
+    """
+    If `quants` is a `Quantity[ndarray]`, but all floats in the ndarray are exactly equal,
+    then return a `Quantity[unit]`, where `unit` is the unit of `quants` (e.g., degC).
+
+    :param quants:
+        Quantity[ndarray]
+    :return:
+        `quants` unchanged if the array has more than one value,
+        otherwise a Quantity with the single shared value
+    """
+    if len(set(quants.m)) == 1:
+        temperatures_cycle1_float = quants.m[0]
+        quants = pint.Quantity(temperatures_cycle1_float, quants.u)
+    return quants
+
+
 def _bsl(x: Iterable[CustomStep] | CustomStep) -> Sequence[CustomStep]:
     return list(x) if isinstance(x, Iterable) else [x]
 
@@ -1625,10 +1668,15 @@ class Protocol(ProtoCommand):
         _set_or_create(qe, "QSLibProtocol").text = str(attr.asdict(self))
         _set_or_create(qe, "QSLibVerson").text = __version__
         _set_or_create(e, "CoverTemperature").text = str(covertemperature)
-        if self.volume is not None:
-            _set_or_create(e, "SampleVolume").text = str(self.volume)
-        if self.runmode is not None:
-            _set_or_create(e, "RunMode").text = str(self.runmode)
+        _set_or_create(e, "SampleVolume").text = str(self.volume or 50)
+        _set_or_create(e, "RunMode").text = str(self.runmode or "Standard")
+        _set_or_create(e, "UserName").text = ""
+        _set_or_create(e, "TubeType").text = "0"
+        _set_or_create(e, "BlockID").text = "18"  # FIXME
+        _set_or_create(e, "Delay").text = "0.0"
+        _set_or_create(e, "ExtendedPCRCycles").text = "0"
+        _set_or_create(e, "ExtendedHoldTemp").text = "0"
+        _set_or_create(e, "ExtendedHoldTime").text = "0"
         if self.filters:
             x = _set_or_create(e, "CollectionProfile", ProfileId="1")
             for f in self.filters:
@@ -1717,6 +1765,41 @@ class Protocol(ProtoCommand):
 
         return True
 
+    def validate(self, fix: bool = True):
+        for i, stage in enumerate(self.stages):
+            if (stage.index is not None) and (stage.index != i + 1):
+                if fix:
+                    log.warn(
+                        "Stage %s is at index %d of protocol, but has set index %d. Fixing.",
+                        stage,
+                        i + 1,
+                        stage.index,
+                    )
+                    stage.index = i + 1
+                else:
+                    raise ValueError(
+                        "Stage %s is at index %d of protocol, but has set index %d."
+                        % (stage, i + 1, stage.index)
+                    )
+
+            if stage.label is not None:
+                m = re.match(r"STAGE_(\d+)", stage.label)
+                if m and (int(m[1]) != i + 1):
+                    if fix:
+                        log.warn(
+                            "Stage %s has label %s, which implies index %d, but is at index %d of protocol. Fixing.",
+                            stage,
+                            stage.label,
+                            int(m[1]),
+                            i + 1,
+                        )
+                        stage.label = f"STAGE_{i+1}"
+                    else:
+                        raise ValueError(
+                            "Stage %s has label %s, which implies index %d, but is at index %d of protocol."
+                            % (stage, stage.label, int(m[1]), i + 1)
+                        )
+
 
 for c in cast(
     Sequence[Protocol],
